/*
 * $Id: android.js 144176 2017-10-02 07:22:32Z gualex $
 * @Copyright OrangeApps
 */

var config = config || {};

var cordovaPlatform = {
	hideSplashScreen: function (data) {
		setTimeout(function () {
			navigator.splashscreen.hide();

			window.addEventListener('' +
				'orientationchange', function () {
				application.orientationChange();
			}, false);

			application.orientationChange();
		}, 1000);
	},

	processBuyItem: function (item) {
		var receipt = JSON.parse(item.transaction.receipt);

		var purchase = {
			orderId: item.transaction.id,
			productId: item.id,
			receipt: item.transaction.receipt,
			signature: item.transaction.signature,
			deviceType: application.deviceType,
			itemPrice: item.priceMicros / 1000000,
			purchaseTime: receipt.purchaseTime,
			itemCurrency: item.currency
		};

		application.executeChild('checkPurchase', purchase);
	},

	appRunning: function (data) {
		var that = this;

		store.ready(function () {
			// переносим данные о ценах во фрейм
			var productsWithState = {};
			_.each(store.products, function (item) {
				if (item.state && item.state != 'invalid' && item.price && item.price.replace) {
					productsWithState[item.id] = {
						itemPrice: item.priceMicros / 1000000.0,
						price: item.price,
						price_currency_code: item.currency,
					};
				}
			});

			application.executeChild('applyAndroidProducts', productsWithState);
		});

		// Продукты джема
		var goodProducts = ['1', '2', '3', '4', '9', '14', '15', '77', '82', '117', '118', '119', '120', '121', '122',
			'124', '125', '126', '1000', '1002', '1003', '1004'];

		var productsToLoad = _.intersection(this._rawProducts, goodProducts) || [];
		this.initBilling(productsToLoad);
	},

	processOwnedProductIfNeeded: function () {
		var approvedProduct = _.find(store.products, function(item) {
			return item.state === 'approved' && item.alias !== 'application data';
		});

		if (!approvedProduct) {
			return;
		}

		// потребляем продукт
		this.processBuyItem(approvedProduct);
	},

	rateThisGame: function (data) {
		var marketWindow = window.open('market://details?id=' + application.appName, '_system');
		setTimeout(marketWindow.close, 1000);

		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent('af_try_rate', {});
		}

		if (window.FirebasePlugin && window.FirebasePlugin.logEvent) {
			window.FirebasePlugin.logEvent('try_rate', {});
		}
	},

	init: function (data) {
		var that = this;

		StatusBar.hide();
		application.appName = data.appName;
		application.appGCMId = data.appGCMId;
		application.social.appKeys = data.appKeys;
		application.social.socialAppIds = data.socialAppIds;

		application.executeChild('deviceInfo', device);

		if (localStorage['Settings']) {
			application.executeChild('setParentStorageSpecial', {
				Settings: myJsonParse(localStorage['Settings'], true)
			});
		}

		// если есть признак gdpr, то передаем его в клиент
		if (localStorage['gdpr']) {
			application.executeChild('setParentStorageSpecial', {
				gdpr: 1
			});
		}
		this._rawProducts = data.products;

		application.initGame = true;
		application.executeChild('init');
	},

	initBilling: function (products) {
		var that = this;

		store.verbosity = store.ERROR;

		_.each(products, function (productId) {
			store.register({
				id: productId,
				alias: productId,
				type: store.CONSUMABLE
			});
		});

		store.error(function (e) {
		});

		store.validator = function (product, callback) {
			callback(true, 'ok');
		};

		store.when('product').approved(function (product) {
			if (product.alias === 'application data') {
				return;
			}

			product.verify();
		});

		store.when('product').verified(function (product) {
			that.processOwnedProductIfNeeded();
		});

		store.validator = function(product, callback) {
			console.log(product);
			callback(true, {}); // success!
		};

		store.refresh();
	},

	buy: function (data) {
		if (navigator.connection.type === 'none') {
			return;
		}

		var itemCurrency = data.product.desc.currency;
		var itemPrice = data.product.priceCurrency[itemCurrency];

		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent('af_initiated_checkout', {
				af_price: itemPrice,
				af_content_id: data.product.product,
				af_currency: itemCurrency
			});
		}

		var product = store.get(data.product.product);
		if (!product) {
			return;
		}

		if (product.state === 'approved') {
			this.processOwnedProductIfNeeded();
			return;
		}

		store.order(product);
	},

	successPurchase: function (data) {
		// нужно отметить товар как полученный
		var product = store.get(data.product);
		if (product && product.state === 'approved') {
			product.finish();

			var itemPrice = product.priceNum;

			if (data.isValid) {
				application.registerPurchase(data.orderId, data.product, itemPrice, data.itemCurrency);
			}
		}
	},

	pause: function () {
		application.orientation = window.orientation;
		application.executeChild('disableSound');
	},

	resume: function () {
		if (!application.initGame) {
			document.location = 'index.html';
			return;
		}
		application.executeChild('orientationChange');
		var toogle = false;
		if (this.musicState) {
			toogle = true;
		}
		application.executeChild('enableSound', toogle);
		application.executeChild('resumeGameApp');
	},

	deviceType: 0,
	messages: [
		{
			type: 'exitApp',
			callback: function () {
				if (window.navigator && window.navigator.app) {
					window.navigator.app.exitApp();
				}
				window.close();
			}
		}
	]
};
