/*
 * $Id: application.js 149116 2017-12-06 12:03:20Z gualex $
 * @Copyright OrangeApps
 */

/* eslint-disable no-console */

var application = {
	messages: [],

	run: function (version) {
		var that = this;

		this.version = version;

		this.baseUrl = baseUrl + relativeFrameUrl;

		StatusBar.hide();
		window.screen.lockOrientation('portrait');

		window.onerror = function (msg, url, line, columnNo, error) {
			console.log('JS Exception: ' + msg + ' URL: ' + url + ' Line: ' + line);
			alert(msg);
			setTimeout(function () {
				location.reload();
			}, 1000);

			return true;
		};

		that._isForeground = true;
		document.addEventListener('resume', function () {
			that._isForeground = true;
			that.platform.resume();
		}, false);

		document.addEventListener('pause', function () {
			that._isForeground = false;
			that.platform.pause();
		}, false);

		document.addEventListener('backbutton', function () {
			if (!that.initGame) {
				return;
			}
			that.executeChild('backbuttonClick');
		}, false);

		this.social = new Social();
		this.platform = cordovaPlatform;

		this.deviceType = cordovaPlatform.deviceType;
		this.orientation = window.orientation;

		this.createMessageProcessor();

		this.getDeviceIdPromise()
			.then(application.getInstallAppsFlyerPromise)
			.then(application.getLocalePromise)
			.then(application.getCreateFramePromise)
			.catch(function (error) {
				var errMessage;
				if (error && error.message) {
					errMessage = error.message;
				} else {
					errMessage = application.getNoConnectionMessage();
				}

				alert(errMessage);
				document.location.reload();
			});

		// предотвращаем масштабирование шрифтов из-за системных настроек
		if (window.MobileAccessibility) {
			window.MobileAccessibility.usePreferredTextZoom(false);
		}

		// Загружаем remoteConfig, не ждем всего остального
		this.loadRemoteConfig();

		// Обходным путем проверяем версию локальных нотификаций,
		// если не 0.9*, то дропаем связь с плагином
		if (cordova.plugins.notification) {
			if (cordova.plugins.notification.local._mergeWithDefaults) {
				var tmp = {}
				cordova.plugins.notification.local._mergeWithDefaults(tmp);
				if (!tmp.meta || tmp.meta.version.indexOf('0.9') !== 0) {
					cordova.plugins.notification = undefined;
				}
			} else {
				cordova.plugins.notification = undefined;
			}
		}
	},

	loadRemoteConfig: function () {
		if (!window.FirebasePlugin ||
			!window.FirebasePlugin.fetch ||
			!window.FirebasePlugin.activateFetched ||
			!window.FirebasePlugin.getValue) {
			return;
		}

		setTimeout(function() {
			var cacheExpirationSeconds = 60 * 60 * 2; // кэшируем результат на 2 часа
			window.FirebasePlugin.fetch(cacheExpirationSeconds, function () {
				console.log('Firebase remote config fetched.');
				window.FirebasePlugin.activateFetched(function() {
					// есть настройки
					registerPrediction();
				}, function () {
					// нет настроек для активации
				});
			}, function () {
				// проблема с fetch
			});
		});

		function registerPrediction() {
			_.each(['predict_spend', 'predict_not_spend'], function (key) {
				if (localStorage[key] !== 'true') {
					window.FirebasePlugin.getValue(key, function(value) {
						if (value === 'true') {
							localStorage[key] = 'true';
							window.plugins.appsFlyer.trackEvent('af_' + key, {});
						}
					});
				}
			});
		}
	},

	sendNonFatalCrash: function (errorPlace, errorMessage) {
	},

	getInstallAppsFlyerPromise: function () {
		var afPromise = new Promise(function (resolve) {
			if (typeof(appsFlyerDevKey) !== 'string') {
				window.plugins.appsFlyer = undefined;
			}

			if (!window.plugins.appsFlyer || window.plugins.appsFlyer.initSdk.length != 3) {
				window.plugins.appsFlyer = undefined;
				resolve();
				return;
			}

			// пробуем версию с объектом
			var args = {
				devKey: appsFlyerDevKey,
				//isDebug: true, // !!!
				onInstallConversionDataListener: true
			};
			if (cordovaPlatform.deviceType == 2 && config.appleId) {
				args.appId = config.appleId;
			}

			window.plugins.appsFlyer.initSdk(args, onSuccess, onError);
			getAppsflyerId();

			function getAppsflyerId() {
				window.plugins.appsFlyer.getAppsFlyerUID(function (id) {
					application.appsflyer_id = id;
					window.plugins.appsFlyer.trackEvent('af_session', {});
					resolve();
				});
			}

			function onSuccess (msg) {
				setTimeout(setupUninstallTracking, 0);
				try {
					var obj = JSON.parse(msg);
					if (obj.data && obj.data.af_status == 'Non-organic') {
						application.appsflyer_data = obj.data;
					}
				} catch (err) {
				}
			}

			function onError(msg) {
			}

			function setupUninstallTracking() {
				// для android
				if (config.firebaseSenderId &&
					application.deviceType === 0 &&
					window.plugins.appsFlyer.enableUninstallTracking &&
					window.plugins.appsFlyer.updateServerUninstallToken &&
					window.FirebasePlugin &&
					window.FirebasePlugin.onTokenRefresh) {

					window.plugins.appsFlyer.enableUninstallTracking(
						config.firebaseSenderId,
						function (message) {
							console.info('enableUninstallTracking', message);
						}, function (error) {
							console.error('enableUninstallTracking', error);
						});

					window.FirebasePlugin.onTokenRefresh(function(token) {
						console.info('Refresh token', token);
						window.plugins.appsFlyer.updateServerUninstallToken(token);
					}, function(error) {
						console.error('onTokenRefresh', error);
					});
				}
			}
		});

		var timeoutPromise = application.delay(500);

		return Promise.race([afPromise, timeoutPromise]);
	},

	getNoConnectionMessage: function () {
		var locale = application.locale || 'ru';
		if (!application.errorText[locale]) {
			locale = 'en';
		}
		return application.errorText[locale][0];
	},

	getDeviceIdPromise: function () {
		return new Promise(function (resolve, reject) {
			if (application.deviceType == 2) {
				window.plugins.uniqueDeviceID.get(function (id) {
					application.uuid = id;
					resolve();
				}, reject);
			} else {
				application.uuid = device.uuid;
				resolve();
			}
		});
	},

	createMessageProcessor: function () {
		var that = this;
		this.messages.push({
			type: 'currentScreen',
			callback: function (data) {
				var page = data.page;
				var win = 'none';
				if (data.window) {
					win = data.window;
				}
				window.currentScreen = page + '-' + win;
			}
		}, {
			type: 'heyzap',
			callback: function (data) {
				if (!window.HeyzapAds) {
					return;
				}

				switch (data.operation) {
					case 'init':
						application.HeyzapInit();
						break;
					case 'preparerewardad':
						application.HeyzapPrepareRewardAd();
						break;
					case 'showrewardad':
						application.HeyzapShowRewardAd();
						break;
				}
			}
		}, {
			type: 'admob',
			callback: function (data) {
				if (!window.admob || !config.adMob || !config.adMob.rewardedVideo) {
					application.executeChild('adMob_disabled');
					return;
				}

				switch (data.operation) {
					case 'prepare':
						application.admobPrepare();
						break;
					case 'show':
						application.admobShow();
						break;
				}
			}
		}, {
			type: 'myTarget_showFullscreen',
			callback: _.bind(that.myTarget_showFullscreen, that)
		}, {
			type: 'myTarget_loadFullscreen',
			callback: _.bind(that.myTarget_loadFullscreen, that)
		}, {
			type: 'callApi',
			callback: _.bind(that.social.callApi, that.social)
		}, {
			type: 'apprequests',
			callback: _.bind(that.social.request, that.social)
		}, {
			type: 'doPublication',
			callback: _.bind(that.social.doPublication, that.social)
		}, {
			type: 'opengraph',
			callback: _.bind(that.social.opengraph, that.social)
		}, {
			type: 'musicState',
			callback: function (data) {
				that.musicState = data.state;
			}
		}, {
			type: 'hideSplashScreen',
			callback: _.bind(that.platform.hideSplashScreen, that.platform)
		}, {
			type: 'officialGroup',
			callback: function (data) {
				window.open(data.url, '_blank', 'location=yes');
			}
		}, {
			type: 'assessmentGooglePlay',
			callback: _.bind(that.platform.rateThisGame, that.platform)
		}, {
			type: 'openSocialLogin',
			callback: _.bind(that.social.openSocialLogin, that.social)
		}, {
			type: 'socialLogout',
			callback: _.bind(that.social.socialLogout, that.social)
		}, {
			type: 'socialLogin',
			callback: _.bind(that.social.socialLogin, that.social)
		}, {
			type: 'init',
			callback: _.bind(that.platform.init, that.platform)
		}, {
			type: 'buy',
			callback: _.bind(that.platform.buy, that.platform)
		}, {
			type: 'successPurchase',
			callback: _.bind(that.platform.successPurchase, that.platform)
		}, {
			type: 'localNotification_schedule',
			callback: _.bind(that.localNotification_schedule, that)
		}, {
			type: 'localNotification_clear',
			callback: _.bind(that.localNotification_clear, that)
		}, {
			type: 'localNotification_set',
			callback: _.bind(that.localNotification_set, that)
		}, {
			type: 'appRunning',
			callback: _.bind(that.appRunning, that)
		}, {
			type: 'removeStorageItems',
			callback: _.bind(that.removeStorageItems, that)
		}, {
			type: 'logAppsflyerEvent',
			callback: _.bind(that.logAppsflyerEvent, that)
		}, {
			type: 'socialLoginNeedsReload',
			callback: function () { location.reload(); }
		}, {
			type: 'setLocalStorage',
			callback: _.bind(that.setLocalStorage, that)
		}, {
			type: 'openUrl',
			callback: _.bind(that.openUrl, that)
		}, {
			type: 'logForCrash',
			callback: _.bind(that.logForCrash, that)
		}, {
			type: 'getRemoteConfig',
			callback: _.bind(that.getRemoteConfig, that)
		}, {
			type: 'firebaseLogEvent',
			callback: _.bind(that.firebaseLogEvent, that)
		});

		for (var key in this.platform.messages){
			this.messages.push(this.platform.messages[key]);
		}

		window.addEventListener('message', function (e) {
			console.log('message: ', e.data);

			var messageData = String(JSON.stringify(e.data)).substring(0, 200);

			// зачем тут перебор?
			for (var key in that.messages) {
				if (that.messages[key].type == e.data.name) {
					that.messages[key].callback.call(null, e.data);
					return;
				}
			}
		});
	},

	executeChild: function (name, params) {
		console.log('executeChild', arguments);

		if (!this.child) {
			return;
		}

		var func;
		if (params !== undefined) {
			func = {name: 'callFunction', functionName: name, params: params};
		} else {
			func = {name: 'callFunction', functionName: name};
		}
		this.child.contentWindow.postMessage(func, '*');
	},

	orientationChange: function () {
		if (this.orientation != window.orientation) {
			this.orientation = window.orientation;
			this.executeChild('orientationChange');
		}
	},

	getLocalePromise: function () {
		return new Promise(function (resolve, reject) {
			application.origLocale = navigator.language;
			application.replaceLocaleToKnown(navigator.language);
			resolve();
		});
	},

	replaceLocaleToKnown: function (lcid) {
		lcid = lcid.toLowerCase();

		// FIXME связать с параметрами сборки
		var supportedLangs = ['en', 'ru', 'de', 'fr', 'it', 'jp'];
		supportedLangs.push('es');
		supportedLangs.push('zht');
		supportedLangs.push('zh');
		// supportedLangs.push('lv');
		// supportedLangs.push('pl');
		// supportedLangs.push('br');

		if (baseUrl.indexOf('stand.tapclap.com') !== -1) {
			// supportedLangs.push('lv');
			// supportedLangs.push('pl');
			// supportedLangs.push('br');
		}

		var lang = replaceInternal(lcid);

		if (supportedLangs.indexOf(lang) != -1) {
			this.locale = lang;
		} else {
			this.locale = supportedLangs[0];
		}

		function replaceInternal(lcid) {
			var lang = lcid.split('-')[0];

			if (lcid === 'pt-br') {
				return 'br';
			}

			if (lcid.indexOf('zh-Hant') === 0) {
				return 'zht';
			}

			if (lcid === 'zh-tw' || lcid === 'zh-hk' || lcid === 'zh-mo') {
				return 'zht';
			}

			switch (lang) {
				case 'hy':
				case 'az':
				case 'be':
				case 'kk':
				case 'ky':
				case 'mo':
				case 'tg':
				case 'tk':
				case 'uz':
				case 'uk':
					return 'ru';

				case 'ja':
					return 'jp';
			}

			return lang;
		}
	},

	getCreateFramePromise: function () {
		var framePromise = new Promise(function (resolve, reject) {
			var src = application.baseUrl.replace('%locale%', application.locale);
			src += '?deviceUid=' + application.uuid +
				'&deviceType=' + application.deviceType +
				'&version=' + application.version +
				'&locale=' + application.locale;

			if (application.appsflyer_id) {
				src += '&appsflyerId=' + encodeURIComponent(application.appsflyer_id);
			}

			src += '&rnd=' + Math.floor(Math.random() * 1000000);

			var child = document.createElement('iframe');
			application.child = child;
			child.onload = resolve;
			child.onerror = reject;
			child.src = src;
			child.width = '100%';
			child.height = '100%';
			child.frameBorder = '0';
			child.style['position'] = 'absolute';
			child.style['top'] = '0px';
			child.style['left'] = '0px';
			document.body.appendChild(child);
		});

		var timeoutPromise = application.delay(60000)
			.then(function () {
				throw new Error(application.getNoConnectionMessage());
			});
		return Promise.race([framePromise, timeoutPromise]);
	},

	delay: function (time) {
		return new Promise(function (fulfill) {
			setTimeout(fulfill, time);
		});
	},

	setLocalStorage: function (data) {
		if (data.key) {
			var value = typeof(data.value) == 'object' ? JSON.stringify(data.value) : undefined;
			localStorage[data.key] = value;
		}
	},

	openUrl: function (data) {
		var method = '_blank';

		if ( data.url.substr(0, 9) == 'market://' ) {
			method = '_system';
		}

		window.open(data.url, method);
	},

	localNotification_schedule: function (data) {
		// на старых apk нет плагина
		if (cordova.plugins.notification && this._isForeground) {
			var id = data.params.id;

			cordova.plugins.notification.local.hasPermission(function (granted) {
				if (!granted) {
					return;
				}
				cordova.plugins.notification.local.isPresent(id, function (present) {
					if (present) {
						cordova.plugins.notification.local.update(data.params);
					} else {
						cordova.plugins.notification.local.schedule(data.params);
					}
				});
			});
		}
	},

	localNotification_clear: function (data) {
		if (cordova.plugins.notification && this._isForeground) {
			cordova.plugins.notification.local.hasPermission(function (granted) {
				if (!granted) {
					return;
				}
				cordova.plugins.notification.local.clear(data.params.id);
			});
		}
	},

	localNotification_set: function (data) {
		if (!cordova.plugins.notification || !this._isForeground || !data.params || !data.params.items) {
			return;
		}

		cordova.plugins.notification.local.hasPermission(function (granted) {
			if (!granted) {
				return;
			}

			_.each(data.params.items, processSingleNotification);
		});

		function processSingleNotification(item) {
			cordova.plugins.notification.local.isPresent(item.id, function (present) {
				if (present) {
					if (item.clear) {
						cordova.plugins.notification.local.clear(item.id);
					} else {
						cordova.plugins.notification.local.update({
							id: item.id,
							title: item.title,
							text: item.text,
							trigger: {at: new Date(item.at)},
						});
					}
				} else {
					cordova.plugins.notification.local.schedule({
						id: item.id,
						title: item.title,
						text: item.text,
						trigger: {at: new Date(item.at)},

						foreground: true,
						icon: 'res://large_notif',
						smallIcon: 'res://small_notif',
						attachments: item.attachments,
					});
				}
			});
		}
	},

	admobPrepare: function () {
		if (!this._admobInitialized) {
			this._admobInitialized = true;
			this.admobInit();
		}

		admob.rewardvideo.prepare();
	},

	admobInit: function () {
		var self = this;

		// Если есть Ogury, то инициализируем его перед admob
		if (window.CPresage) {
			CPresage.presageSdk.initializeSdk(
				function (event) {
					console.log('Ogury init success: ', event);
				},
				function (error) {
					console.error('Ogury init error: ', error);
				},
				config.oguryKey
			);
		}

		admob.rewardvideo.config({
			id: config.adMob.rewardedVideo,
			isTesting: false,
			autoShow: false
		});

		var hasReward = false;

		document.addEventListener('admob.rewardvideo.events.LOAD_FAIL', function(event) {
			console.log('LOAD_FAIL', event);
			self.executeChild('adMob_noAd');
		});

		document.addEventListener('admob.rewardvideo.events.LOAD', function(event) {
			console.log('LOAD', event);
			self.executeChild('adMob_adLoaded');
		});

		document.addEventListener('admob.rewardvideo.events.CLOSE', function(event) {
			if (hasReward) {
				self.executeChild('adMob_takeReward');
				self.reportAds();
			} else {
				self.executeChild('adMob_noReward');
			}
			hasReward = false;
			console.log('CLOSE', event)
		});

		document.addEventListener('admob.rewardvideo.events.REWARD', function(event) {
			console.log('REWARD', event);
			hasReward = true;
		});
	},

	admobShow: function () {
		admob.rewardvideo.show();
	},

	HeyzapInit: function () {
		var that = this;
		HeyzapAds.start(heyzapPublisherKey).then(function () {
			HeyzapAds.IncentivizedAd.addEventListener(HeyzapAds.IncentivizedAd.Events.COMPLETE, function (tag) {
				//screen.unlockOrientation();
				//fixSound();
				that.executeChild('heyzapShowAdSuccess');
				that.reportAds();

				//navigator.notification.alert("Heyzap on complete");
			});
			HeyzapAds.IncentivizedAd.addEventListener(HeyzapAds.IncentivizedAd.Events.INCOMPLETE, function (tag) {
				//screen.unlockOrientation();
				//fixSound();
				that.executeChild('heyzapShowAdError');
				//navigator.notification.alert("Heyzap on incomplete");
			});
			HeyzapAds.IncentivizedAd.addEventListener(HeyzapAds.IncentivizedAd.Events.AVAILABLE, function (tag) {
				that.executeChild('heyzapPrepareNextAdSuccess');
				that._heyzapLastPrepare = Date.now();
				//navigator.notification.alert("Heyzap on available");
			});
			that.executeChild('heyzapInitSuccess');
		}, function (error) {
			console.log(error);
			//navigator.notification.alert(error);
		});
	},

	HeyzapPrepareRewardAd: function () {
		HeyzapAds.IncentivizedAd.fetch().then(function () {
			//navigator.notification.alert("Prepare reward ad complete");
		}, function (error) {
			console.log(error);
			//navigator.notification.alert(error);
		});
	},

	HeyzapShowRewardAd: function () {
		var that = this;
		HeyzapAds.IncentivizedAd.show().then(function () {
			//navigator.notification.alert("Show reward ad complete");
		}, function (error) {
			that.executeChild('heyzapShowAdError');
			//navigator.notification.alert(error);
		});
	},

	myTarget_loadFullscreen: function (data) {
		var self = this;
		if (window.MyTarget && window.MyTarget.loadFullscreen && data.slotId) {
			if (self._myTargetLastResult == 'success' && this._myTargetLastResultData) {
				application.executeChild('myTargetSuccess', this._myTargetLastResultData);
				return;
			}

			if (self._myTargetLastResult == 'error' && this._myTargetLastResultData) {
				application.executeChild('myTargetError', this._myTargetLastResultData);
				return;
			}

			window.MyTarget.loadFullscreen(data.slotId, function (success) {
				self._myTargetLastResult = 'success';
				self._myTargetLastResultData = success;
				application.executeChild('myTargetSuccess', success);
			}, function (error) {
				self._myTargetLastResult = 'error';
				self._myTargetLastResultData = error;
				application.executeChild('myTargetError', error);
			});
		} else {
			application.executeChild('myTargetError', 'ads not supported');
		}
	},

	myTarget_showFullscreen: function () {
		var self = this;
		this._myTargetLastResult = null;
		this._myTargetLastResultData = null;
		if (window.MyTarget && window.MyTarget.showFullscreen) {
			window.MyTarget.showFullscreen(function (success) {
				application.executeChild('myTargetSuccess', success);
				self.reportAds();
			}, function (error) {
				application.executeChild('myTargetError', error);
			});
		} else {
			application.executeChild('myTargetError', 'ads not supported');
		}
	},

	appRunning: function (data) {
		this.phonegapNetwork = data.phonegapNetwork;
		if (data && data.viewerId) {
			this.viewerId = data.viewerId;
		}

		if (data.phonegapNetwork == 'facebook') {
			if (data.daysInGame === 7 && !localStorage['loginOn7day']) {
				facebookConnectPlugin.logEvent('loginOn7day');
				localStorage['loginOn7day'] = true;
			}

			if (data.daysInGame === 30 && !localStorage['loginOn30day']) {
				facebookConnectPlugin.logEvent('loginOn30day');
				localStorage['loginOn30day'] = true;
			}

			if (data.episode == 5 && !localStorage['onEpisode5']) {
				facebookConnectPlugin.logEvent('onEpisode5');
				localStorage['onEpisode5'] = true;
			}
		}

		if (this.platform.appRunning) {
			this.platform.appRunning(data);
		}

		// регистрируем логины в appsFlyer
		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent('af_login', {});
		}

		this.pushInit(data.episode);
	},

	pushInit: function (episode) {
		if (!window.FirebasePlugin) {
			return;
		}

		if (application.deviceType === 2 && episode >= 2) {
			// попробуем тут опереться на разрешение для плагина firebase

			// У юзеров могут быть разрешены старые токены.
			// Тут не только запрос разрешения у пользователя, но синхронизация токенов APN
			window.FirebasePlugin.grantPermission();
		}

		// обрабатываем параметры локальных нотификаций
		if (cordova.plugins.notification &&
			cordova.plugins.notification.local.launchDetails &&
			cordova.plugins.notification.local.launchDetails.action === 'click')
		{
			FirebaseLogEvent('local_push_open', {
				id: String(cordova.plugins.notification.local.launchDetails.id)
			});
		}

		// пока обрабатываем только foreground клики
		if (cordova.plugins.notification) {
			cordova.plugins.notification.local.on('click', function (notification, state) {

			});

			// Эта штука заработает только если до deviceready (в apk)
			// предотвратить раннее срабатывание событий window.skipLocalNotificationReady
			cordova.plugins.notification.local.fireQueuedEvents();
		}
	},

	removeStorageItems: function (data) {
		for (var key in data.items) {
			localStorage.removeItem(data.items[key]);
		}
	},

	registerPurchase: function (orderId, productId, itemPrice, itemCurrency) {
		// регистрируем покупку в ОК
		if (application.phonegapNetwork == 'odnoklassniki') {
			SocialOk.reportPayment(orderId, itemPrice, itemCurrency, _.noop, _.noop);
		}

		// регистрируем покупку в appsFlyer
		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent('af_purchase', {
				af_revenue: itemPrice,
				af_content_id: productId,
				af_currency: itemCurrency
			});
		}
	},

	logForCrash: function (data) {

	},

	logAppsflyerEvent: function (data) {
		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent(data.eventName, data.params);
		}

		FirebaseLogEvent(data.eventName, {});
	},

	reportAds: function () {
		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.trackEvent('af_showads', {});
		}
	},

	getRemoteConfig: function (data) {
		if (!window.FirebasePlugin || !window.FirebasePlugin.getValue || !data.params) {
			return;
		}

		var resultParams = {};
		var keys = _.keys(data.params);
		var finishFunc = _.after(keys.length, function () {
			application.executeChild('applyRemoteConfig', resultParams);
		});

		_.each(keys, function (key) {
			window.FirebasePlugin.getValue(key, function(value) {
				if (value  !== undefined && value !== '') {
					resultParams[key] = value;
				}
				finishFunc();
			}, function (error) {
				console.error(error);
				finishFunc();
			});
		});
	},

	firebaseLogEvent: function (data) {
		FirebaseLogEvent(data.event_name, data.event_params);
	},

	errorText: {
		ru: [
			'Невозможно запустить приложение, проверьте подключение к интернету!',
			'Не удалось проверить платёж, необходимо соединение с интернетом!'
		], en: [
			'Unable to start the application, check the the Internet connection!',
			'Failed to verify the payment, you must have an Internet connection!'
		], jp: [
			'アプリケーションを起動することができません、インターネット接続を確認！',
			'お支払いを確認できませんでした、あなたがインターネットに接続している必要があります！'
		], br: [
			'Unable to start the application, check the the Internet connection!',
			'Failed to verify the payment, you must have an Internet connection!'
		], lv: [
			'Unable to start the application, check the the Internet connection!',
			'Failed to verify the payment, you must have an Internet connection!'
		], pl: [
			'Unable to start the application, check the the Internet connection!',
			'Failed to verify the payment, you must have an Internet connection!'
		]
	}
};

function myJsonParse(s, silent) {
	if (typeof(s) == 'object') {
		return s;
	}

	try {
		return JSON.parse(s);
	} catch (e) {
		if (silent === true) {
			// return empty object
			return {};
		} else {
			throw Error('Error parse in: ' + String(s).substring(0, 100));
		}
	}
}

var Social = function () {
	this.noPublicOpengraph = false;
	this.login = {
		vkontakte: false,
		odnoklassniki: false,
		facebook: false
	};
};

Social.prototype.callApi = function (data) {
	console.log('callApi', data);

	var that = this;
	switch (data.network) {
		case 'facebook':
			var path = data.path;
			if (data.params !== undefined) {
				path = path + '?';
			}
			path = path.substr(1);

			var first = true;
			for (var key in data.params) {
				if (!first) {
					path += '&';
				}
				path = path + key + '=' + data.params[key];
				first = false;
			}
			this.facebookApi(path, data.success);
			break;
		case 'vkontakte':
			if (this.login.vkontakte) {
				this.vkApi(data.method, data.params, data.success);
			} else {
				this.vkInit(function () {
					that.login.vkontakte = true;
					that.vkApi(data.method, data.params, data.success);
				});
			}
			break;
		case 'odnoklassniki':
			var method = data.params.method;
			delete data.params['method'];
			if (that.login.odnoklassniki) {
				this.okApi(method, data.params, data.success);
			} else {
				this.okInit(function () {
					that.login.odnoklassniki = true;
					that.okApi(method, data.params, data.success);
				});
			}
			break;
	}
};

Social.prototype.request = function (data) {
	console.log(data);
	var that = this;
	switch (data.network) {
		case 'facebook':
			// https://github.com/jeduan/cordova-plugin-facebook4/issues/521
			data.params.actionType = '';

			facebookConnectPlugin.showDialog(data.params, function (response) {
				if (data.success) {
					application.executeChild('waitingSuccess');
				}
			}, function (err) {
				console.log('FB apps.sendRequest err', err);
				if (data.success) {
					var repeat = false;
					if (typeof(err) != 'object') {
						repeat = true;
						console.log('typeof: !object');
					}
					application.executeChild('cancelExternalRequests', repeat);
				}
			});
			break;

		case 'vkontakte':
			if (this.login.vkontakte) {
				this.vkRequest(data.uids, data.type, data.message, data.success);
			} else {
				this.vkInit(function () {
					that.login.vkontakte = true;
					that.vkRequest(data.uids, data.type, data.message, data.success);
				});
			}
			break;

		case 'odnoklassniki':
			this.okRequest(data.params.message, data.params.uids, data.success, data.isInvite);
			break;
	}
};

Social.prototype.doPublication = function (data) {
	switch (data.network) {
		case 'facebook':
			facebookConnectPlugin.showDialog(data.options, function (response) {
				if (data.success) {
					application.executeChild('waitingSuccess');
				}
			}, function (e) {
			});
			break;

		case 'odnoklassniki':
			this.okPublish(data);
			break;
	}
};

// может это встроить в facebookOpengraph ?
Social.prototype.opengraph = function (data) {
	if (!this.noPublicOpengraph) {
		if (data.path) {
			this.facebookOpengraphPath(data);
		} else {
			this.facebookOpengraph(data);
		}
	}
};

Social.prototype.facebookLogin = function (callback) {
	var that = this;
	facebookConnectPlugin.login(['public_profile', 'user_friends'],
		function (data) {

			var userData = {};
			userData.phonegapNetwork = 'facebook';
			userData.userId = data.authResponse.userID;
			userData.accessToken = data.authResponse.accessToken;
			application.executeChild('sdkLogin', userData);

			if (window.plugins.appsFlyer) {
				window.plugins.appsFlyer.setAppUserId(String(userData.userId));
			}

			if (callback) {
				callback();
			}
		},
		function (error) {
			console.log('error login ', error);
			if (error.errorCode == '190') {
				that.facebookLogin(callback);
			} else {
				application.executeChild('resetNetwork');
				localStorage['phonegapNetwork'] = undefined;
			}

			// User cancelled dialog, errorCode = 4201
			var skipLog = error.errorCode == '4201' || error == 'User cancelled.';

			if (!skipLog) {
				application.sendNonFatalCrash('facebookLogin', (error.errorMessage || error));
			}
		});
};

Social.prototype.facebookOpengraph = function (str) {
	var that = this;
	facebookConnectPlugin.getAccessToken(function (token) {
		var parse = str.url.split('?');
		var graph = parse.shift();
		parse = parse.join('?');
		var url = parse.split('=');
		var type = url.shift();
		url = url.join('=');
		var data = {'fb:explicitly_shared': true, access_token: token};
		if (type != 'achievement') {
			graph = graph + '?' + type + '=' + encodeURIComponent(url);
		} else {
			data[type] = url;
		}
		$.ajax({
			type: 'POST',
			url: 'https://graph.facebook.com' + graph,
			dataType: 'json',
			data: data,
			error: function (e) {
				console.log('err ', e);

				var skipLog = e.status == 403;
				if (!skipLog) {
					application.sendNonFatalCrash('facebookOpengraph', JSON.stringify(e));
				}
				/*
				 {
				 "readyState": 4,
				 "responseText": "...",
				 "responseJSON": {
				 "error": {
				 "message": "(#200) Requires extended permission: publish_actions",
				 "type": "OAuthException",
				 "code": 200,
				 "fbtrace_id": "..."
				 }
				 },
				 "status": 403,
				 "statusText": "Forbidden"
				 }
				 */

				if (that.noPublicOpengraph) {
					return;
				}

				facebookConnectPlugin.login(['publish_actions'],
					function (d) {
						that.facebookOpengraph(str);
					}, function (e) {
						that.noPublicOpengraph = true;
					});
			},
			success: function (response) {
			}
		});
	}, function (e) {
	});
};

Social.prototype.facebookOpengraphPath = function (data) {
	var that = this;
	facebookConnectPlugin.getAccessToken(function (token) {
		var params = _.clone(data.params);
		var path = data.path;
		params['fb:explicitly_shared'] = true;
		params['access_token'] = token;

		$.ajax({
			type: 'POST',
			url: 'https://graph.facebook.com' + path,
			dataType: 'json',
			data: params,
			error: function (e) {
				console.log('err ', e);

				var skipLog = e.status == 403;
				if (!skipLog) {
					application.sendNonFatalCrash('facebookOpengraphPath', JSON.stringify(e));
				}

				if (that.noPublicOpengraph) {
					return;
				}

				facebookConnectPlugin.login(['publish_actions'],
					function (d) {
						that.facebookOpengraphPath(data);
					}, function (e) {
						that.noPublicOpengraph = true;
					});
			},
			success: function (response) {
			}
		});
	}, _.noop);
};

Social.prototype.facebookApi = function (path, success) {
	var that = this;

	facebookConnectPlugin.api(path, [], function (response) {
		if (success) {
			application.executeChild('waitingSuccess', response);
		}
	}, function (e) {
		// {errorMessage: "An active access token must be used to query information about the current user.", errorCode: "2500"}
		// разные версии плагинов возвращают ошибки в разном формате
		var needRelogin = e.errorCode == 190 || e.errorCode == 2500 || e == 'You are not logged in.';
		if (needRelogin) {
			that.facebookLogin(function () {
				application.child.src = application.child.src; // eslint-disable-line no-self-assign
			});
			return;
		}
		console.log('facebookApi error ', e);
		application.sendNonFatalCrash('facebookApi', JSON.stringify(e));
	});
};

Social.prototype.vkInit = function (callback) {

	SocialVk.init(this.socialAppIds['vkontakte'], function () {
		callback();
	}, function (err) {
		console.log('VK plugin', err);
		application.sendNonFatalCrash('vkInit', JSON.stringify(err));
	});
};

Social.prototype.vkLogin = function (callback) {
	SocialVk.login(['wall', 'friends', 'offline', 'notify', 'groups'],
		function (data) {
			var userData = {};

			// для ios и android SocialVk.login возвращает результаты в разном формате
			data = myJsonParse(data);
			userData.userId = data.user.id || data.user[0].id;

			userData.phonegapNetwork = 'vkontakte';
			userData.accessToken = data.token;
			application.executeChild('sdkLogin', userData);

			if (window.plugins.appsFlyer) {
				window.plugins.appsFlyer.setAppUserId(String(userData.userId));
			}

			if (callback) {
				callback();
			}
		}, function (err) {
			application.executeChild('resetNetwork');
			localStorage['phonegapNetwork'] = undefined;

			console.log('VK login', err);
			var skipError = err && err.indexOf('(Canceled)') !== -1;
			if (!skipError) {
				application.sendNonFatalCrash('vkLogin', JSON.stringify(err));
			}
		});
};

Social.prototype.vkApi = function (method, params, success) {
	var that = this;
	SocialVk.callApiMethod(method, params, function (response) {

		try {
			response = myJsonParse(response);
		} catch (err) {
			// пришел не JSON, а какой-то шлак
			return;
		}

		if (response.items) {
			response = response.items;
		}
		response = {response: response};
		if (success) {
			application.executeChild('waitingSuccess', response);
		}
	}, function (error) {
		console.log('error ', error);

		var skipError =
			(error.indexOf('code: 15') !== -1) ||
			(error.indexOf('(Canceled)') !== -1);
		if (!skipError) {
			application.sendNonFatalCrash('vkApi', JSON.stringify(error));
		}

		if (error.indexOf('code: 5') > 0 || error.indexOf('code: 113') > 0) {
			that.vkLogin(function () {
				location.reload();
			});
		}
		if (error.indexOf('code: 15;') > 0) {
			if (success) {
				application.executeChild('waitingSuccess', {response: 'ok'});
			}
		}
	});
};

Social.prototype.vkRequest = function (uid, type, message, success) {
	var that = this;
	console.log('type ', type, ' message ', message);
	SocialVk.callApiMethod('apps.sendRequest', {user_id: uid, text: message, type: type}, function (response) {
		console.log('VK call API apps.sendRequest', response);
		if (success) {
			application.executeChild('waitingSuccess', response);
		}
	}, function (err) {
		console.log('VK apps.sendRequest err', err);

		// "VKError (API errorVKError (code: 15; ; Access denied: user should have this application installed))"
		// "VKError (API errorVKError (code: 15; ; Access denied: it is forbidden to send requests to this user))"
		// "VKError (Canceled)"
		var skipError =
			(err.indexOf('code: 15') !== -1) ||
			(err.indexOf('(Canceled)') !== -1);
		if (!skipError) {
			application.sendNonFatalCrash('vkRequest', JSON.stringify(err));
		}

		if (success) {
			var repeat = false;
			if (err.indexOf('code: 15') > 0) {
				repeat = true;
			}
			application.executeChild('cancelExternalRequests', repeat);
		}
	});
};

Social.prototype.okInit = function (callback) {
	// Метод работает криво, даже если this.appKeys['odnoklassniki'] не задан все равно ОК
	SocialOk.init(this.socialAppIds['odnoklassniki'], '', this.appKeys['odnoklassniki'], function () {
		if (callback) {
			callback();
		}
	}, function (err) {
		console.log('OK plugin', err);
		application.sendNonFatalCrash('okInit', JSON.stringify(err));
	});
};

Social.prototype.okLogin = function (callback) {
	SocialOk.login('LONG_ACCESS_TOKEN,VALUABLE_ACCESS,APP_INVITE,PUBLISH_TO_STREAM', function (data) {
		// Тут могут прислать ошибку

		console.log(data);
		data = myJsonParse(data);
		var userData = {};
		userData.phonegapNetwork = 'odnoklassniki';
		userData.userId = data.user.uid;
		userData.accessToken = data.token;
		application.okToken = data.token;
		application.okSecretKey = data['session_secret_key'];
		application.executeChild('sdkLogin', userData);

		if (window.plugins.appsFlyer) {
			window.plugins.appsFlyer.setAppUserId(String(userData.userId));
		}

		var time = new Date().getTime();
		SocialOk.callApiMethod('sdk.reportStats', {
			stats: JSON.stringify({
				time: time,
				version: '1',
				stats: [{
					time: time,
					type: 'launch',
					id: 'launch',
					data: ['1']
				}]
			})
		},
		function (data) {},
		function (e) { console.log(e); }
		);

		if (callback) {
			callback();
		}
	}, function (err) {
		application.executeChild('resetNetwork');
		localStorage['phonegapNetwork'] = undefined;
		console.log('OK login', err);

		// "OK login error: Authorization was canceled"
		// "OK login error: Авторизация была отменена"
		var skipLog =
			(String(err).indexOf('canceled') !== -1) ||
			(String(err).indexOf('отмен') !== -1);
		if (!skipLog) {
			application.sendNonFatalCrash('okLogin', JSON.stringify(err));
		}
	});
};

Social.prototype.okApi = function (method, params, success) {
	var that = this;
	SocialOk.callApiMethod(method, params, function (response) {
		console.log('response ', response);

		try {
			response = myJsonParse(response);
		} catch (err) {
			// пришел не JSON, а какой-то шлак
			return;
		}

		if (parseInt(response.error_code) == 102 || parseInt(response.error_code) == 10 && method != 'notifications.sendFromUser') {
			that.okLogin(function () {
				that.login.odnoklassniki = true;
				that.okApi(method, params, success);
			});
			return;
		} else if (method == 'notifications.sendFromUser') {
			application.executeChild('waitingSuccess', true);
		} else if (success) {
			application.executeChild('cancelExternalRequests', response);
		}
		if (success) {
			application.executeChild('waitingSuccess', response);
		}
	}, function (err) {
		console.log('okApi err ', err);
		var sErr = JSON.stringify(err);
		application.sendNonFatalCrash('okApi', sErr);

		// Error Domain=ru.ok.sdk Code=7 \"No access_token defined you should invoke authorizeWithPermissions first
		// Error Domain=ru.ok.api Code=10 \"PERMISSION_DENIED : User must grant an access to permission 'VALUABLE_ACCESS'\"
		if (sErr.indexOf('Code=7 ') != -1 || sErr.indexOf('Code=102 ') != -1 || sErr.indexOf('Code=10 ')) {

			if (SocialOk.logout) {
				SocialOk.logout();
			}

			that.okLogin(function () {
				that.login.odnoklassniki = true;
				that.okApi(method, params, success);
			});
		} else if (method == 'notifications.sendFromUser') {
			application.executeChild('waitingSuccess', {});
		}
	});
};

Social.prototype.okPublish = function (data) {
	if (data.attachment &&
		SocialOk.version)
	{
		var onSuccess = function (res) {
			application.executeChild('waitingSuccess');
		};

		SocialOk.performPosting(
			{'st.attachment': JSON.stringify(data.attachment)},
			onSuccess
		);
		return;
	}

	var token = '';
	var that = this;
	if (application.okToken) {
		token = '&token=' + application.okToken + '&secret=' + application.okSecretKey;
	} else {
		this.okLogin(function () {
			that.okPublish(data);
		});
		return;
	}

	var postWindow = window.open(data.url + token, '_blank', 'location=yes');
	postWindow.addEventListener('loadstart', function (q) {
		console.log(q);
		var url = q.url;
		if (url.indexOf('dk?cmd=WidgetMediatopicPost') > 0) {
			postWindow.close();
			if (data.success) {
				application.executeChild('waitingSuccess');
			}
		}
	});
};

Social.prototype.okRequest = function (message, uids, success, isInvite) {
	if (SocialOk.version) {
		var params = {
			'st.comment': message
		};

		if (uids) {
			if (typeof uids === 'string') {
				params['st.target'] = uids;
			} else {
				params['st.target'] = JSON.stringify(uids);
			}
			params['st.target_only'] = 'on';
		}

		var onSuccess = function (res) {
			var selected = JSON.parse(res.selected);
			application.executeChild('waitingSuccess', selected);
		};

		var onError = function () {
			console.log('error', arguments);
		};

		if (isInvite) {
			SocialOk.performInvite(params, onSuccess, onError);
		}

		if (!isInvite) {
			SocialOk.performSuggest(params, onSuccess, onError);
		}

		return;
	}

	var token = '';
	var that = this;
	if (application.okToken) {
		token = '&token=' + application.okToken + '&secret=' + application.okSecretKey;
	} else {
		this.okLogin(function () {
			that.okRequest(message, uids, success, isInvite);
		});
		return;
	}

	var widgetName = isInvite ? 'WidgetInvite' : 'WidgetSuggest';
	var beginUrl = application.baseUrl.split('/').slice(0, -2).join('/');
	var path = beginUrl + '/odnoklassniki/widgetinvite.php?widget_id=' + widgetName + '&comment=';
	var url = path + encodeURIComponent(message) + '&uids=' + uids;

	var postWindow = window.open(url + token, '_blank', 'location=yes');

	postWindow.addEventListener('loadstart', function (q) {
		console.log(q);
		var url = q.url;
		if (url.indexOf('?code=OK') > 0) {
			if (success) {
				application.executeChild('waitingSuccess');
			}
		}

		if (url.indexOf('?code=') > 0) {
			postWindow.close();
		}
	});
};

Social.prototype.openSocialLogin = function (data) {
	var that = this;

	switch (data.network) {
		case 'facebook':
			this.facebookLogin();
			break;
		case 'vkontakte':
			this.vkInit(this.vkLogin);
			break;
		case 'odnoklassniki':
			this.okInit(this.okLogin);
			break;
		default:
			this.socialLogin = window.open(data.url, '_blank', 'location=yes');
			this.socialLogin.addEventListener('loadstart', function (q) {
				var url = q.url;
				if (url.indexOf('_auth.php?') > 0) {
					that.socialLogin.close();
					that.socialLogin = document.createElement('iframe');
					that.socialLogin.src = url;
					that.socialLogin.width = '100%';
					socialLogin.height = '100%';
					that.socialLogin.frameBorder = '0';
					that.socialLogin.style['position'] = 'absolute';
					that.socialLogin.style['top'] = '0px';
					socialLogin.style['left'] = '0px';
					document.body.appendChild(that.socialLogin);
				}
			});

			break;
	}
};

Social.prototype.socialLogout = function (data) {
	switch (data.network) {
		case 'facebook':
			facebookConnectPlugin.logout(_.noop, _.noop);
			break;

		case 'vkontakte':
			SocialVk.logout(_.noop, _.noop);
			break;

		case 'odnoklassniki':
			if (SocialOk.logout) {
				SocialOk.logout(_.noop, _.noop);
			}
			break;
	}
};

Social.prototype.socialLogin = function (data) {
	var str = data.loginData.callServiceUrl.split(':');
	if (str[0] == 'http') {
		data.loginData.callServiceUrl = data.loginData.callServiceUrl.replace('http', 'https');
	}

	localStorage['call_service_url'] = data.loginData.callServiceUrl;
	localStorage['session_data'] = JSON.stringify(data.loginData.sessionData);
	localStorage['social_network'] = data.loginData.phonegapNetwork;

	if (data.loginData) {
		application.child.contentWindow.postMessage({
			name: 'callFunction',
			functionName: 'socialLoginSuccess',
			params: data.loginData
		}, '*');
	} else {
		application.executeChild('resetNetwork');
	}
	if (this.socialLogin.remove) {
		this.socialLogin.remove();
	}
};

function FirebaseLogEvent(event, params) {
	if (window.FirebasePlugin && window.FirebasePlugin.logEvent) {
		window.FirebasePlugin.logEvent(event, params);
	}
}

var heyzapPublisherKey = 'ec6dbd77468329bdd5b35247546fd550'; // новый heyzap